#include "AMAengine.h"
#include "mex.h"

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    // CHECK THE NUMBER OF INPUTS
    if (nrhs != 3)
        mexErrMsgTxt ("not enough inputs");

    // GET THE FILTER RESPONSES, r
    int ndims = mxGetNumberOfDimensions (prhs[0]); // get # of dimensions
    if (ndims != 2)
        mexErrMsgTxt ("r must be 2d");
    const int *dims = mxGetDimensions (prhs[0]);   // actual dimensions
    int mr = dims[0]; // r is m x n
    int nr = dims[1];
	double *r = mxGetPr (prhs[0]); // get pointer to elements

    // GET THE FILTER RESPONSE STDDEV, sigma
    ndims = mxGetNumberOfDimensions (prhs[1]); // get # of dimensions
    if (ndims != 2)
        mexErrMsgTxt ("sigma must be 2d");
    dims = mxGetDimensions (prhs[1]);   // actual dimensions
    int ms = dims[0]; // s is m x n
    int ns = dims[1];
    if (ms != mr || ns != nr)
        mexErrMsgTxt ("r and sigma must be the same size");
	double *s = mxGetPr (prhs[1]); // get pointer to elements

    // GET CATEGORY INDEX. c
    ndims = mxGetNumberOfDimensions (prhs[2]); // get # of dimensions
    if (ndims != 2)
        mexErrMsgTxt ("category index must be 2d");
    dims = mxGetDimensions (prhs[2]);   // actual dimensions
    int mc = dims[0]; // c is m x 1
    int nc = dims[1];
    if (mc != mr || nc != 1)
        mexErrMsgTxt ("category index must be jx1");
	double *c = mxGetPr (prhs[2]); // get pointer to elements

    // ALLOCATE mr x 1 MATRIX TO CONTAIN POSTERIOR PROB OF CORRECT CTG
    int ret1dims[2] = { mr, 1 };
    plhs[0] = mxCreateNumericArray (2, ret1dims, mxDOUBLE_CLASS, mxREAL);
    // save address of allocated memory into the return value
	double *p = mxGetPr (plhs[0]);
    
    // ALLOCATE mr x nCtg MATRIX TO CONTAIN POSTERIOR PROB OF ALL CTG 
    const int nCtg = *std::max_element (&c[0], &c[0] + mr);
    int ret2dims[2] = { mr, nCtg };
    plhs[1] = mxCreateNumericArray (2, ret2dims, mxDOUBLE_CLASS, mxREAL);
    // save address of allocated memory into the return value
	double *pAll = mxGetPr (plhs[1]);

    // POSTERIOR PROB OF CORRECT (p) & ALL (pAll) CATEGORIES FOR EACH STIMULUS
    AMAposteriors (mr, nr, r, s, c, p, pAll);
}
